%QBMMGC  Quadratic Boolean MM-based optimization tool
%
% L = OBMM(UE, PI, PE)
%
% J. Bioucas-Dias @ 2008
% To be used for research purposes only.
%
% Other Copyright Acknowledgements
% This .m file is a modified version of a code authored by
% Oliver Woodford and the following copyright holds
% Copyright (c) 2007-2008, University of Oxford
% 
%
% see:   C. Rother, V. Kolmogorov, V. Lempitsky, and M. Szummer
%		 Optimizing binary MRFs via extended roof duality
%		 CVPR 2007.
%
%        Y. Boykov, V. Kolmogorov
%		 An Experimental Comparison of Min-Cut/Max-Flow Algorithms for Energy Minimization in Vision
%		 PAMI, 26(9):1124-1137, September 2004.
%        
%        P. L. Hammer, P. Hansen, and B. Simeone. 
% 		 Roof duality, complementation and persistency in quadratic 0-1 optimization. 
% 		 Mathematical Programming, 28:121155, 1984.
% 
% 		 E. Boros, P. L. Hammer, and X. Sun.
% 		 Network flows and minimization of quadratic pseudo-Boolean functions. 
% 		 Technical Report RRR 17-1991, RUTCOR Research Report, May 1991.



% Majorization Minimization Quadratic Boolean Optimization Based on Graph
% Cuts (by J. Bioucas-Dias).
%
% Replaces submodular edges with a sequence of Mojorizer sub-modular
% functions
%
% IN:
%   UE - 2xM matrix of unary terms for M nodes, or M = UE(1) if numel(UE)
%        == 1 (assumes all unary terms are zero). UE can be any type, but
%        PE and TE must be of the same type; also, optimality is not
%        guaranteed for non-integer types.
%   PI - {2,3}xN uint32 matrix, each column containing the following
%        information on an edge: [start_node end_node
%        [pairwise_energy_table_index]]. If there are only 2 rows then
%        pairwise_energy_table_index is assumed to be the column number,
%        therefore you must have N == P.
%   PE - 4xP pairwise energy table, each column containing the energies
%        [E00 E01 E10 E11] for a given edge.
%   x  - Initial labeling

%   options - 2x1 cell array. options{1} is a 1x4 uint32 vector of optional
%             parameters:
%      FirstNNodes - Only labels of nodes 1:FirstNNodes will be output.
%                    Also limits nodes considered in QPBO-P and QPBO-I.
%                    Default: M.
%      ImproveMethod - 0: Unlabelled nodes not improved; 1: QPBO-I
%                      employed, assuming preferred label is L0. 2: QPBO-I
%                      employed, using user-defined labelling returned by
%                      callback_func.
%      ContractIters - Number of iterations of QPBO-P to do. Default: 0.
%      LargerInternal - 0: input type also used internally; 1: code uses a
%                       larger representation (i.e. more bits) for energies
%                       than that provided, reducing the possibility of
%                       edges becoming saturated. Default: 0.
%             options{2} - callback_func, a function name or handle which,
%                          given the labelling L (the first output of this
%                          function), returns a logical array of the same
%                          size defining a labelling to be transfromed by
%                          QPBO-I.
%
% OUT:
%   L - 1xFirstNNodes int32 matrix of the energy minimizing state of each
%       node. Labels {0,1} are the optimal label (in a weak sense), while
%       labels < 0 indicate unlabelled nodes. For unlabelled nodes, if L(x)
%       == L(y) then x and y are in the same "strongly connected region".
%   stats - 3x1 int32 vector of stats:
%      stats(1) - number of unlabelled pixels
%      stats(2) - number of strongly connected regions of unlabelled pixels
%      stats(3) - number of unlabelled pixels after running QPBO-P

% $Id: vgg_qpbo.m,v 1.1 2007/12/07 11:27:56 ojw Exp $

%function [L stats] = QPBOPmex(UE,PI,PE,varargin)
function [x, stat,energy] = QBMMGC_v0(UE, PI, PE)

small = 1e-8;

stat=[0,0,0]';

n = size(UE,2);
nUE = prod(size(UE));
nPE = prod(size(PE));
x = zeros(n,1);

% compute energy;
lin_indexUE = (1:2:nUE);
lin_indexPE = (1:4:nPE);


non_regular_terms = (PE(1,:)+PE(4,:)-PE(3,:)-PE(2,:));
mask = non_regular_terms >= 0;
non_regular_terms = (non_regular_terms+small).*mask;

PEM = PE;
% labeling p=0,q=0
PEM(2,:) = PEM(2,:)+non_regular_terms/2;
PEM(3,:) = PEM(3,:)+non_regular_terms/2;

x = double(QPBOPmex(UE, PI, PEM));

% compute energy;
indexPE = (1 + 2*x(PI(1,:))+ x(PI(2,:)))';
energy = sum(UE(lin_indexUE+(x'))) + sum(PE(lin_indexPE+(indexPE-1)));




